@@ -1,7 +1,7 @@
 # serialiser.py - PSKC file parsing functions
 # coding: utf-8
 #
-# Copyright (C) 2016-2017 Arthur de Jong
+# Copyright (C) 2016-2018 Arthur de Jong
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -35,9 +35,11 @@ def my_b64encode(value):
 
 
 class PSKCSerialiser(object):
+    """Class for serialising a PSKC structure to PSKC 1.0 XML."""
 
     @classmethod
     def serialise_file(cls, pskc, output):
+        """Write the PSKC structure to the specified output file."""
         xml = tostring(cls.serialise_document(pskc))
         try:
             output.write(xml)
@@ -47,8 +49,8 @@ class PSKCSerialiser(object):
 
     @classmethod
     def serialise_document(cls, pskc):
-        container = mk_elem('pskc:KeyContainer', Version=pskc.version,
-                            Id=pskc.id)
+        """Convert the PSKC structure to an element tree structure."""
+        container = mk_elem('pskc:KeyContainer', Version='1.0', Id=pskc.id)
         cls.serialise_encryption(pskc.encryption, container)
         cls.serialise_mac(pskc.mac, container)
         for device in pskc.devices:
@@ -57,6 +59,7 @@ class PSKCSerialiser(object):
 
     @classmethod
     def serialise_encryption(cls, encryption, container):
+        """Provide an XML element tree for the encryption information."""
         if all(x is None
                for x in (encryption.id, encryption.key_name, encryption.key,
                          encryption.derivation.algorithm)):
@@ -72,6 +75,7 @@ class PSKCSerialiser(object):
 
     @classmethod
     def serialise_key_derivation(cls, derivation, encryption_key, key_names):
+        """Provide an XML structure for the key derivation properties."""
         derived_key = mk_elem(encryption_key, 'xenc11:DerivedKey', empty=True)
         key_derivation = mk_elem(derived_key, 'xenc11:KeyDerivationMethod',
                                  Algorithm=derivation.algorithm)
@@ -91,25 +95,31 @@ class PSKCSerialiser(object):
 
     @classmethod
     def serialise_mac(cls, mac, container):
-        if not mac.algorithm and not mac.key:
+        """Provide an XML structure for the encrypted MAC key."""
+        key_value = getattr(mac, '_key', None) or mac.pskc.encryption.key
+        if not mac.algorithm and not key_value:
             return
         mac_method = mk_elem(
             container, 'pskc:MACMethod', Algorithm=mac.algorithm, empty=True)
+        if not key_value:
+            return
+        # encrypt the mac key if needed
+        if not hasattr(key_value, 'get_value'):
+            key_value = EncryptedValue.create(mac.pskc, key_value)
+        # construct encrypted MACKey
+        algorithm = key_value.algorithm or mac.pskc.encryption.algorithm
+        cipher_value = key_value.cipher_value
+        if mac.pskc.encryption.iv:
+            cipher_value = mac.pskc.encryption.iv + cipher_value
         mac_key = mk_elem(mac_method, 'pskc:MACKey', empty=True)
-        mk_elem(
-            mac_key, 'xenc:EncryptionMethod',
-            Algorithm=mac.pskc.encryption.algorithm)
+        mk_elem(mac_key, 'xenc:EncryptionMethod', Algorithm=algorithm)
         cipher_data = mk_elem(mac_key, 'xenc:CipherData', empty=True)
-        if mac.key_cipher_value:
-            mk_elem(cipher_data, 'xenc:CipherValue',
-                    base64.b64encode(mac.key_cipher_value).decode())
-        elif mac.key_plain_value:
-            mk_elem(cipher_data, 'xenc:CipherValue',
-                    base64.b64encode(mac.pskc.encryption.encrypt_value(
-                        mac.key_plain_value)).decode())
+        mk_elem(cipher_data, 'xenc:CipherValue',
+                base64.b64encode(cipher_value).decode())
 
     @classmethod
     def serialise_key_package(cls, device, container):
+        """Provide an XML structure for key package."""
         key_package = mk_elem(container, 'pskc:KeyPackage', empty=True)
         if any(x is not None
                for x in (device.manufacturer, device.serial, device.model,
@@ -134,8 +144,9 @@ class PSKCSerialiser(object):
 
     @classmethod
     def serialise_key(cls, key, key_package):
+        """Provide an XML structure for the key information."""
         key_elm = mk_elem(key_package, 'pskc:Key', empty=True, Id=key.id,
-                          Algorithm=key.algorithm, )
+                          Algorithm=key.algorithm)
         mk_elem(key_elm, 'pskc:Issuer', key.issuer)
         if any((key.algorithm_suite, key.challenge_encoding,
                 key.response_encoding, key.response_length)):
@@ -169,6 +180,7 @@ class PSKCSerialiser(object):
 
     @classmethod
     def serialise_data(cls, key, field, key_elm, tag):
+        """Provide an XML structure for the key material."""
         value = getattr(key, '_%s' % field, None)
         pskc = key.device.pskc
         # skip empty values
@@ -196,20 +208,25 @@ class PSKCSerialiser(object):
             mk_elem(element, 'pskc:PlainValue', value2text(value))
         else:
             # encrypted value
+            algorithm = value.algorithm or pskc.encryption.algorithm
+            cipher_value = value.cipher_value
+            if pskc.encryption.iv:
+                cipher_value = pskc.encryption.iv + cipher_value
             encrypted_value = mk_elem(
                 element, 'pskc:EncryptedValue', empty=True)
             mk_elem(encrypted_value, 'xenc:EncryptionMethod',
-                    Algorithm=value.algorithm)
+                    Algorithm=algorithm)
             cipher_data = mk_elem(
                 encrypted_value, 'xenc:CipherData', empty=True)
             mk_elem(cipher_data, 'xenc:CipherValue',
-                    base64.b64encode(value.cipher_value).decode())
+                    base64.b64encode(cipher_value).decode())
             if value.mac_value:
                 mk_elem(element, 'pskc:ValueMAC',
                         base64.b64encode(value.mac_value).decode())
 
     @classmethod
     def serialise_policy(cls, policy, key_elm):
+        """Provide an XML structure with the key policy information."""
         # check if any policy attribute is set
         if not policy.key_usage and all(x is None for x in (
                 policy.start_date, policy.expiry_date,
@@ -235,6 +252,7 @@ class PSKCSerialiser(object):
 
     @classmethod
     def serialise_signature(cls, signature, container):
+        """Provide an XML structure for embedded XML signature."""
         if not signature.key:
             return container
         # move the namespace to the root element and reformat before signing
